#!/usr/local/bin/perl -w
#------------------------------------------------------------------------------
# Licensed Materials - Property of IBM (C) Copyright IBM Corp. 2010, 2010
# All Rights Reserved US Government Users Restricted Rights - Use, duplication
# or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
#------------------------------------------------------------------------------

#  perl itcam_yn_teps.pl [saveexport.sql type file]
#
#  john alvord, IBM Corporation, 22 May 2016
#  jalvord@us.ibm.com
#
# tested on
# This is perl 5, version 16, subversion 3 (v5.16.3) built for MSWin32-x86-multi-thread
# (with 1 registered patch, see perl -V for more detail)
# $DB::single=2;   # remember debug breakpoint
#
my $gVersion = 0.52000;

#use warnings::unused; # debug used to check for unused variables
use strict;
use warnings;

use Data::Dumper;               # debug only


my $opt_v;
my $opt_atr;
my $opt_workpath;

while (@ARGV) {
   if ($ARGV[0] eq "-v") {
      shift(@ARGV);
      $opt_v = 1;
   } elsif ( $ARGV[0] eq "-atr") {
      shift(@ARGV);
      $opt_atr = shift(@ARGV);
      die "option -atr with no following atr specification\n" if !defined $opt_atr;
   } elsif ( $ARGV[0] eq "-workpath") {
      shift(@ARGV);
      $opt_workpath = shift(@ARGV);
      die "option -workpath with no following workpath specification\n" if !defined $opt_workpath;
   } else {
     last;
   }
}
die "no saveexport-type file supplied" if !defined $ARGV[0];
my $savexfn = $ARGV[0];
shift(@ARGV);

die "Unknown argument options" if $#ARGV != -1;

die "file $savexfn not found" if ! -e $savexfn;

$opt_v = 0 if !defined $opt_v;

$opt_atr = "kyn.atr" if !defined $opt_atr;
$opt_workpath = "" if !defined $opt_workpath;


# following list is the "old" ITCAM YN attibutes, first 32 characters.
# If the table name in the KFWQUERY matches one in the kyn.atr file and
# the AFFINITIES first 32 characters match the following entries, an UPDATE SQL will be created.
my %affx = (
              # section for YN
              # old ITCAM YN attributes hashed to new ITCAM attributes
              # based on kyn.atr.7.2.0.0.0
              "00000000000000000000000000000010"  =>  "%IBM.STATIC186          0000000001000000000",  #002
              "00000000000000000000000000000010"  =>  "%IBM.STATIC186          0000000004000000000",  #005
              "0000m0000000000000000000000000I1"  =>  "&IBM.CAM_WAS            0000000001000000000",  #008
              "0000m0000000000000000000000000I1"  =>  "&IBM.CAM_WAS            0000000002000000000",  #016
              "0000m0000000000000000000000000I1"  =>  "&IBM.CAM_WAS            0000000003000000000",  #021
              "0000G000000000000000000000000021"  =>  "&IBM.CAM_ESB            0000000002000000000",  #025
              "0000W000000000000000000000000000"  =>  "%IBM.STATIC025          0000000002000000000",  #029
              "0000m000000000000000000000000021"  =>  "&IBM.CAM7_WAS           0000000001000000000",  #032
              "0000m000000000000000000000000021"  =>  "&IBM.CAM7_WAS           0000000004000000000",  #035
              "0000G000000000000000000000000001"  =>  "&IBM.CAM_SCA            0000000002000000000",  #038
              "000000000000000000000000000000G0"  =>  "%IBM.STATIC182          0000000002000000000",  #041

              # Following is an older TEPS construction which is no longer supported
              "%IBM.IDMULTIPLE         00000000"  =>  " ",

              # section for YJ to be added later

              # section for HT to be added later
           );

my $aff32;               # first 32 characters to checking %afff table

my %tableag = ();        # Table => Application Group unused at moment
my %tableaff = ();       # Table => Affinities

# Stage 1, read the attribute file and extract table, affinities, attribute group name and store away

my $oneline;
my $ll = 0;
my $state = 0;
my $iattribute_group;
my $iaffinities;
my $itable;
open(AFF, "<$opt_atr") || die("Could not open attribute file $opt_atr\n");
while ($oneline = <AFF>) {
   chop $oneline;
   $ll += 1;
   # Example lines
   # name WebSphere_XS_Configuration.Origin_Node
   # affi "%IBM.CAM_WXS_ZONE       000000000700000000"
   if ($state == 0) {                                     # looking for Origin_Node line
      $oneline =~ /name (\S+)\.Origin_Node/;
      next if !defined $1;
      $iattribute_group = $1;
      $state = 1;

   } elsif ($state == 1) {                                # looking for affi or Affinities line
      next if substr($oneline,0,4) ne "affi";
      $oneline =~ /\"(.*)\"/;
      $iaffinities = $1;
$DB::single=2 if !defined $iaffinities;
      if (!defined $iaffinities) {                        # Some are within double quotes, others are plain
         $oneline =~ /affi (.*)/;
         $iaffinities = $1;
      }
$DB::single=2 if !defined $iaffinities;
      $iaffinities .= "0" if length($iaffinities) == 42;  # some are 42 characters long and *must* be extended to 43 bytes
      $state = 2;

   } elsif ($state == 2) {                                # looking for tabl entry
      next if substr($oneline,0,4) ne "tabl";
      $oneline =~ /tabl (.*)/;
      $itable = $1;
$DB::single=2 if !defined $itable;
      $itable =~ s/\s+$//;                              # strip trailing blanks
      $iattribute_group =~ s/\s+$//;                    # strip trailing blanks
      $tableag{$itable} = $iattribute_group;            # record data in hash
      $tableaff{$itable} = $iaffinities;                # ...
      $state = 0;                                       # look for next Origin_Node
   }
}
close(IN1);

# log output
my $logfn = "itcamyn_teps.log";
open(LOG, "> $logfn") || die("Could not open log $logfn\n");
my $logline;
print LOG "ITCAM_YP_TEPS processing log $gVersion\n";

# corrective SQL output
my $sqlfn = "itcamyn_teps.sql";
open(SQL, "> $sqlfn") || die("Could not open sql file $sqlfn\n");
my $sqlline;


my $iid;                       # data recorded from INSERT - ID
my $ilastuser;                 #                             LASTUSER
my $imodified;                 #                             MODIFIED
my $iquery;                    #                             QUERY


my $advi = -1;                 # Advisories to shown in log
my @adv = ();



# we have a series of INSERT SQL statements that look like this
#
# INSERT INTO KFWQUERY ( ID, APPL, AFFINITIES, PROD_PROV, NAME, TEXT, LASTUSER, MODIFIED, QUERY ) VALUES ( 'KYN.0310.KYNMSGENG', 'KYN', '&IBM.CAM_ESB            0000000002000000000', '*', 'Kyn:KYNV611068', 'Kyn:KYNV611069', 'IBM', '1140531004709000', 'table1.name=KYNMSGENG@table2.name=KYNMSGQUE@tableCount=2@readonly=T@' );
# INSERT INTO KFWQUERY ( ID, APPL, AFFINITIES, PROD_PROV, NAME, TEXT, LASTUSER, MODIFIED, QUERY ) VALUES ( 'KYN.0240.KYNCACHE', 'KYN', '&IBM.CAM_WAS            0000000001000000000', '*', 'Kyn:KYN1006', 'Kyn:KYN1007', 'IBM', '1140531004709000', 'table1.name=KYNCACHE@tableCount=1@readonly=T@' );
#
# extract them into a useful Perl structure
#
   $ll = 0;
my $columns_txt;
my $values_txt;
my @columns;
my %colx;
my $cx;
my $ix;
my @values;
my $colct;
open(IN1, "< $savexfn") || die("Could not open input $savexfn\n");
while ($oneline = <IN1>) {
   chop $oneline;
   $ll += 1;
   next if substr($oneline,0,20) ne "INSERT INTO KFWQUERY";   # only worry about KFWQUERY, skip otherwise
   print "working on $ll\n" if $opt_v == 1;                   # could be useful for debugging
   $oneline =~ /\((.*?)\) VALUES \((.*)\)/;                  # extract out columns and value
   $columns_txt = $1;
   $values_txt = $2;
   @columns = ();
   @values = ();
   $columns_txt =~ s/\ //g;
   @columns = split(",",$columns_txt);                        # columns converted into an array
   $colct = $#columns;                                        # number of columns
   for (my $i=0;$i<=$colct;$i++){                             # set up hash of column name to index
      $colx{$columns[$i]} = $i;
   }
   # Following is borrowed from http://www.perlmonks.org/?node_id=552860
   # The logic handles all the cases of single/double quotes and commas within quotes
$DB::single=2 if $ll == 32;
   push(@values, $+) while $values_txt =~ m{ \s*(
       # groups the phrase inside double quotes
       "([^\"\\]*(?:\\.[^\"\\]*)*)"\s*,?
       # groups the phrase inside single quotes
       | '([^\'\\]*(?:\\.[^\'\\]*)*)'\s*,?
       # trims leading/trailing space from phrase
       | ([^,\s]+(?:\s+[^,\s]+)*)\s*,?
       # just to grab empty phrases
       | (),
       )\s*}gx;
   push(@values, undef) if $values_txt =~ m/,\s*$/;

   # Start by gathering data for the log
   $cx = $colx{"QUERY"};
   $iquery = $values[$cx];
$DB::single=2 if !defined $iquery;
   # query column starts like this
   #tableCount=1@table1.name=KYNGCACT@table1.column1
   $iquery =~ /tableCount\=(\d+)\@/;
   $iquery =~ /table1.name\=(\S+?)@/;
   my $itable = $1;
   next if substr($itable,0,3) ne "KYN";              # ignore queries unless for YN Agent
   my $newaff = $tableaff{$itable};                   # See what new AFFINITIES should be present for this table
   if (!defined $newaff) {                            # if unknown, that means an older unused table so ignore ot
      $advi++;$adv[$advi] = "unknown table $itable at line $ll";
      next;
   }

   $ix = $colx{"AFFINITIES"};                         # record AFFINITIES
   $iaffinities = $values[$ix];
   next if $iaffinities eq $newaff;                   # if current AFFINITIES is what should be present, all is well go to next
   $aff32 = substr($iaffinities,0,32);                # Check first 32 characters, which avoids the character based release in old affinities string
   next if !defined $affx{$aff32};                    # if no match, skip. Maybe and an advisory?
   $ix = $colx{"ID"};                                 # record ID
   $iid = $values[$ix];
   $ix = $colx{"LASTUSER"};                           # record LASTUSER
   $ilastuser = $values[$ix];
   $ix = $colx{"MODIFIED"};                           # record MODIFIED
   $imodified = $values[$ix];
   $logline = "KFWQUERY ";                            # compose logline
   $logline .= $iid . " ";
   $logline .= $ilastuser . " ";
   $logline .= $imodified . " ";
   $logline .= "old_aff=\"" . $iaffinities . "\" ";
   $logline .= "new_aff=\"" . $newaff . "\" ";
   print LOG "$logline\n";                            # write to log

   # example form of UPDATE SQL statement
   # UPDATE O4SRV.TSITDESC
   # SET AUTOSTART = '*NO'
   # WHERE SITNAME = 'Posible_maquina_apagada';
   $sqlline = "UPDATE KFWQUERY ";
   $sqlline .= "SET AFFINITIES = \'" . $newaff . "\' ";
   $sqlline .= "WHERE ID = \'" . $iid . "\'; ";
   print SQL "$sqlline\n";                            # write to SQL
}

if ($advi != 0) {                                     # if any advisories, print them
   print LOG "\n";
   print LOG "Advisories\n";
   for (my $a=0;$a<=$advi;$a++) {
      print LOG "$adv[$a]\n";
   }
}

# close up shop and go home

close(IN1);
close(SQL);
close(LOG);
exit 0;
# 0.52000  : new script
